local _, Addon = ...
local Battle = C_PetBattles
local Ability = Addon.Ability

local Pets = Addon:NewModule('BattlePets')
local TypeIcons = 'Interface/PetBattles/PetIcon-'
local Player = LE_BATTLE_PET_ALLY


--[[ Static ]]--

function Pets:Get(owner, index)
	local pet = {
		owner = owner,
		index = index,
	}
	
	return setmetatable(pet, self)
end

function Pets:GetCurrent(owner)
	local index = Battle.GetActivePet(owner)
	return self:Get(owner, index)
end

function Pets:GetNum(owner)
	return Battle.GetNumPets(owner)
end


--[[ Display ]]--

function Pets:GetDisplayInfo()
	local name, specieName = self:GetDisplayNames()
	return name, specieName,
		self:GetIcon(),
		self:GetTypeIcon(),
		self:GetModel()
end

function Pets:GetDisplayNames()
	local name, specieName = self:GetName()
	return name, specieName ~= name and specieName or ''
end

function Pets:GetTypeIcon()
	return TypeIcons .. PET_TYPE_SUFFIX[self:GetType()]
end

function Pets:GetStats()
	return self:GetHealth(),
		self:GetMaxHealth(),
		self:GetPower(),
		self:GetSpeed(),
		self:GetLevel()
end


--[[ Abilities ]]--

function Pets:GetAbility(i)
	return Ability:New(self, i)
end

function Pets:GetAbilityType(i)
	local type, isHealing = select(7, self:GetAbilityInfo(i))
	return not isHealing and type
end

function Pets:GetModifier(type)
	return type and Battle.GetAttackModifier(type, self:GetType()) or 1
end


--[[ Other ]]--

function Pets:Exists()
	return self:GetNum(self.owner) >= self.index
end

function Pets:IsAlive()
	return self:GetHealth() > 0
end

function Pets:Swap()
	if self.owner == Player and Battle.CanPetSwapIn(self.index) then
		Battle.ChangePet(self.index)
		return true
	end
end

function Pets:GetType()
	return self:GetPetType()
end

function Pets:GetSpecie()
	return self:GetPetSpeciesID()
end

function Pets:GetModel()
	return self:GetDisplayID()
end

function Pets:__index(key)
	return Pets[key] or function(self, ...)
		local func = Battle[key]
		assert(func, key .. ' not found.')
		
		return func(self.owner, self.index, ...)
	end
end